/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2019 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.jpeek.web;

import java.io.IOException;
import org.cactoos.Func;
import org.cactoos.func.IoCheckedFunc;
import org.takes.Response;
import org.takes.rs.RsWithType;

/**
 * Typed pages.
 *
 * <p>There is no thread-safety guarantee.
 *
 * @author Yegor Bugayenko (yegor256@gmail.com)
 * @version $Id: 568de251dc116a33f89b807ee2c47123d841f487 $
 * @since 0.8
 * @checkstyle ClassDataAbstractionCouplingCheck (500 lines)
 * @checkstyle JavadocTagsCheck (500 lines)
 */
final class TypedPages implements Func<String, Response> {

    /**
     * Origin.
     */
    private final Func<String, Response> origin;

    /**
     * Ctor.
     * @param func The func
     */
    TypedPages(final Func<String, Response> func) {
        this.origin = func;
    }

    @Override
    public Response apply(final String path) throws IOException {
        String type = "text/plain";
        if (path.endsWith(".html")) {
            type = "text/html";
        } else if (path.endsWith(".xml")) {
            type = "application/xml";
        } else if (path.endsWith(".svg")) {
            type = "image/svg+xml";
        }
        return new RsWithType(
            new IoCheckedFunc<>(this.origin).apply(path), type
        );
    }

}
