/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2019 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.jpeek;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import org.hamcrest.Matchers;
import org.junit.Test;
import org.llorllale.cactoos.matchers.Assertion;

/**
 * Test case for {@link DefaultBase}.
 * @author Yegor Bugayenko (yegor256@gmail.com)
 * @version $Id: ae62c6725397b0a66e08f8b6beccc49f9b2e8058 $
 * @since 0.1
 * @checkstyle JavadocMethodCheck (500 lines)
 * @checkstyle JavadocTagsCheck (500 lines)
 */
public final class DefaultBaseTest {

    @Test
    public void listsFiles() throws IOException {
        final Path temp = Files.createTempDirectory("");
        temp.resolve("a/b/c").toFile().mkdirs();
        Files.write(
            temp.resolve("a/b/c/x.java"), "Hello".getBytes(),
            StandardOpenOption.CREATE_NEW
        );
        Files.write(
            temp.resolve("a/z.class"), "".getBytes(),
            StandardOpenOption.CREATE_NEW
        );
        new Assertion<>(
            "Must be more the 2 files",
            new DefaultBase(temp).files(),
            Matchers.iterableWithSize(Matchers.greaterThan(2))
        ).affirm();
    }

}
